/* grpix.c
 *
 * this program demonstrates some of Turbo C's
 * graphics pixel manipulation functions
 *
 * requires the TC BGI drivers in the current
 * directory or an environment variable, BGI,
 * that points to it, e.g.:
 *
 * BGI=d:\tc\bgi
 *
 */

#include<graphics.h>
#include<stdio.h>
#include<stdlib.h>
#include<conio.h>
#include<time.h>
#include<string.h>

#if !defined(TRUE)
#define TRUE 1
#endif
#if !defined(FALSE)
#define FALSE 0
#endif


#define atoh(c) (unsigned)((c > '9') \
                ? (c - 'A' + 0xA) : (c-'0'))

#define MAXLOGOROWS 10
char *cobblogo_pixelmap[MAXLOGOROWS] =
    {
{"000FFFF000F000FFFF000F00FFFFF000F00FFFFF000"},
{"00FFFFFF00F00FFFFFF00F00FF00FF00F00FF00FF00"},
{"00FF00FF00F00FF00FF00F00FF00FF00F00FF00FF00"},
{"00FF000000F00FF00FF00F00FFFFFF00F00FFFFFF00"},
{"00FF000000F00FF00FF00F00FFFFF000F00FFFFF000"},
{"00FF000000F00FF00FF00F00FF00FF00F00FF00FF00"},
{"00FF00FF00F00FF00FF00F00FF00FF00F00FF00FF00"},
{"00FFFFFF00F00FFFFFF00F00FFFFFF00F00FFFFFF00"},
{"000FFFF000F000FFFF000F00FFFFF000F00FFFFF000"},
{NULL}
    };

#define BLACKRECTSTARTCOL       100
#define BLACKRECTSTARTROW       100
#define BLACKRECTENDCOL         109
#define BLACKRECTENDROW         210

#define BLUERECTSTARTCOL    (BLACKRECTSTARTCOL-2)
#define BLUERECTSTARTROW    (BLACKRECTSTARTROW-2)
#define BLUERECTENDCOL      (BLACKRECTENDCOL + 2 \
                            +BLACKRECTOFFSET(3))
#define BLUERECTENDROW      (BLACKRECTENDROW+2)

#define WHITERECTSTARTCOL   BLACKRECTSTARTCOL
#define WHITERECTSTARTROW   BLACKRECTSTARTROW
#define WHITERECTENDCOL     (BLACKRECTENDCOL + \
                            BLACKRECTOFFSET(3))
#define WHITERECTENDROW     BLACKRECTENDROW

#define BLACKRECTOFFSET(n)  (n*11)

int maxx, maxy, maxcolor;
long maxpix,quarterpix;

void main(void);
void error_exit(int errcode, char *message);
void pause(void);
void grmessage(char *message);
void gprintf(char *fmt,...);
void draw_randomly_colored_pixels(void);
void fill_screen_with_colored_pixels(void);
void draw_lines_from_center(void);
void draw_colored_line(unsigned from_x,
    unsigned from_y,unsigned to_x,unsigned to_y);
void exploding_filled_square(unsigned radius,
    unsigned color);
void decay(long numpix,unsigned color);
void draw_pixelmap(unsigned startcol,
    unsigned startrow, char **pixelmap);
void draw_filled_rectangle(unsigned startcol,
    unsigned startrow,unsigned endcol,
    unsigned endrow,unsigned color);

void main(void)
    {
    int graphdriver = DETECT,graphmode;
    char *bgipath, errmsg[80];

        /* get BGI path if one  */
    bgipath = getenv("BGI");

        /* initialize graphics system*/
    initgraph(&graphdriver,&graphmode,bgipath);
    if(graphdriver < 0)
        {
        sprintf(errmsg,"initgraph (%s)",
            grapherrormsg(graphdriver));
        error_exit(graphdriver,errmsg);
        }

	/* initialize random number generator  */
    randomize();
        /* initialize global variables          */
    maxx = getmaxx();
    maxy = getmaxy();
    maxcolor = getmaxcolor();
    maxpix = (long)maxx*maxy;
    quarterpix = maxpix/4;
    setcolor(BLACK);

    draw_randomly_colored_pixels();
    pause();

    cleardevice();
    fill_screen_with_colored_pixels();
    pause();

    draw_lines_from_center();
    pause();

    exploding_filled_square(50,BLACK);
    pause();
    exploding_filled_square(50,CYAN);
    pause();

    decay(maxpix/12L,BLACK);
    pause();

        /* draw COBB Group Logo:                */
        /* first, the blue background           */
    draw_filled_rectangle(BLUERECTSTARTCOL,
            BLUERECTSTARTROW,BLUERECTENDCOL,
            BLUERECTENDROW,BLUE);
        /* next, the white logo background      */
    draw_filled_rectangle(WHITERECTSTARTCOL,
            WHITERECTSTARTROW,WHITERECTENDCOL,
            WHITERECTENDROW,WHITE);

        /* then, each black column              */
    draw_filled_rectangle(BLACKRECTSTARTCOL+
            BLACKRECTOFFSET(0),BLACKRECTSTARTROW,
            BLACKRECTENDCOL+BLACKRECTOFFSET(0),
            BLACKRECTENDROW,BLACK);
    draw_filled_rectangle(BLACKRECTSTARTCOL+
            BLACKRECTOFFSET(1),BLACKRECTSTARTROW,
            BLACKRECTENDCOL+BLACKRECTOFFSET(1),
            BLACKRECTENDROW,BLACK);
    draw_filled_rectangle(BLACKRECTSTARTCOL+
            BLACKRECTOFFSET(2),BLACKRECTSTARTROW,
            BLACKRECTENDCOL+BLACKRECTOFFSET(2),
            BLACKRECTENDROW,BLACK);
    draw_filled_rectangle(BLACKRECTSTARTCOL+
            BLACKRECTOFFSET(3),BLACKRECTSTARTROW,
            BLACKRECTENDCOL+BLACKRECTOFFSET(3),
            BLACKRECTENDROW,BLACK);
        /* finally, the COBB pixelmap           */
    draw_pixelmap(100,103,
            (char **)cobblogo_pixelmap);
    pause();

        /* switch back to text mode, cleanup*/
    closegraph();
    }

    /* displays error #, message and exits
     */
void error_exit(int errcode, char *message)
    {
    printf("Error %d returned from %s\n",errcode,
        message);
    exit(0);
    }

    /* graphics compatible version of printf    */
void gprintf(char *fmt,...)
    {
    char workbuffer[250];
    va_list argptr;

    va_start(argptr,fmt);
    vsprintf(workbuffer,fmt,argptr);
    va_end(arg_ptr);

    grmessage(workbuffer);
    }

    /* displays the message at the left-most
       column at the bottom of the screen, then
       waits for a keypress
     */
void pause(void)
    {
    char *str = " press any key to continue...";

    grmessage(str);
    }

    /* graphics compatible message function     */
void grmessage(char *message)
    {
#ifdef OLD
    unsigned i, j;
    char workbuf[100];

        /* create blank buffer                  */
    j = (strlen(message) < sizeof(workbuf)-1) ?
            strlen(message) : (sizeof(workbuf)-1);

    for(i = 0; i < j; i++)
        workbuf[i] = ' ';
    workbuf[i] = '\0';

        /* write blanks                         */
    outtextxy(0,getmaxy()-(textheight(workbuf)*2),
            workbuf);
#endif
    bar(0,getmaxy()-(textheight(message)*2),textwidth(message),
        getmaxy()-(textheight(message)));
        /* write message                        */
    outtextxy(0,getmaxy()-(textheight(message)*2),
            message);
        /* wait for keypress                    */
    getch();
    }

    /* randomly draws randomly colored pixels   */
void draw_randomly_colored_pixels(void)
    {
    unsigned color,x,y;
    long l;

        /* for 1/4 the pixels on screen         */
    for( l = 0L; l < quarterpix; l++)
        {
            /* get coordinates and color        */
        x = random(maxx+1);
        y = random(maxy+1);
        color = random(maxcolor+1);
            /* write pixel                      */
        putpixel(x,y,color);
        if(kbhit())
            break;
        }
    }

    /* fills screen with each color             */
void fill_screen_with_colored_pixels(void)
    {
    unsigned color,x,y;
    long m,n;

    n = maxpix/maxcolor;

        /* for each color available     */
    for(color = 1, x = y = 0; color <= maxcolor;
            color++)
        {
        for(m = 0L; m < n; m++)
            {
            /* write the pixel          */
            putpixel(x,y,color);
            x++;    /* next column          */
            /* if last column, reset, next row  */
            if(x > maxx)
                {
                x = 0;
                y++;
                }
            /* if on last line, break           */
            if(y > maxy)
                break;
            if(kbhit())
                break;
            }
        if(kbhit())
            break;
        }
    }

    /* draws lines from center pixel            */
void draw_lines_from_center(void)
    {
    unsigned centerx = maxx/2,centery = maxy/2;

    draw_colored_line(centerx-100,centery-100,
            centerx,centery);
    draw_colored_line(centerx,centery-100,
            centerx,centery);
    draw_colored_line(centerx+100,centery-100,
            centerx,centery);
    draw_colored_line(centerx-100,centery,
            centerx,centery);
    draw_colored_line(centerx+100,centery,
            centerx,centery);
    draw_colored_line(centerx-100,centery+100,
            centerx,centery);
    draw_colored_line(centerx,centery+100,
            centerx,centery);
    draw_colored_line(centerx+100,centery+100,
            centerx,centery);
    }

    /* draws line from coordinates              */
void draw_colored_line(unsigned Fx,unsigned Fy,
        unsigned Tx,unsigned Ty)
    {
    unsigned pixel,i,j,k,l,up = FALSE;

        /* if drawing -> \ or / */
    if(Tx >= Fx)
        {
        i = Fx;
        k = Tx;
        j = Fy;
        l = Ty;
        if(Ty < Fy)     /* drawing /    */
            up = TRUE;
        }
    else    /* else drawing <- \ or /   */
        {
        i = Tx;
        k = Fx;
        j = Ty;
        l = Fy;
        if(Ty > Fy)     /* drawing /    */
            up = TRUE;
        }
    while(TRUE)
        {
            /* get the pixel    */
        pixel = getpixel(i,j);
            /* adjust color     */
        if(pixel > 7)
            pixel -= 6;
        else
            pixel += 7;
            /* write pixel      */
        putpixel(i,j,pixel);
            /* increment (decrement) if needed */
        if(j != l)
            if(up)
                j--;
            else
                j++;
        if(i != k)
            i++;
            /* when to and from are equal   */
        if(j == l && i == k)
            break;
        }
    }

    /* concentric squares from center pixel     */
void exploding_filled_square(unsigned radius,
    unsigned color)
    {
    unsigned centerx = maxx/2,
            centery = maxy/2, i, j;

        /* write center pixel   */
    putpixel(centerx,centery,color);

        /* for each square      */        
    for(i = 0; i < radius; i++)
        {
            /* write top line       */
        for(j = centerx-i; j < centerx+i; j++)
            putpixel(j,centery-i,color);
            /* write sides          */
        for(j = centery-i; j < centery+i; j++)
            {
            putpixel(centerx-i,j,color);
            putpixel(centerx+i,j,color);
            }
            /* write bottom         */
        for(j = centerx-i; j <= centerx+i; j++)
            putpixel(j,centery+i,color);
        if(kbhit())
            break;
        }
    }

    /* randomly fills display with pixels       */
void decay(long numpix,unsigned color)
    {
    long l;
    unsigned x,y;

    for( l = 0L; l < numpix; l++)
        {
            /* get coordinates          */
        x = random(maxx+1);
        y = random(maxy+1);
            /* write 4 pixels of color  */
        putpixel(x,y,color);
        putpixel(x+1,y,color);
        putpixel(x,y+1,color);
        putpixel(x+1,y+1,color);
        if(kbhit())
            break;
        }
    }

    /* draws filled rectangle in coordinates    */
void draw_filled_rectangle(unsigned startcol,
        unsigned startrow,unsigned endcol,
        unsigned endrow, unsigned color)
    {
    unsigned i;

    for( ; startrow <= endrow; startrow++)
        for(i = startcol; i <= endcol; i++)
            putpixel(i,startrow,color);
    }

    /* draws pixelmap at coordinates            */
void draw_pixelmap(unsigned startcol,
        unsigned startrow,char **pixelmap)
    {
    unsigned i, j;
    char *p;

        /* for each pointer in the array    */
    for(i = 0; pixelmap[i]; i++)
        {
            /* for each byte in the string      */
        for( p = pixelmap[i], j = 0; p[j]; j++)
            /* display the converted pixel      */
            putpixel(startcol+j,startrow+i,
                    atoh(p[j]));
        }
    }




